/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

 
package net.sourceforge.jedit.pluginholder; 

//pluginholder support
import net.sourceforge.jedit.pluginholder.backend.*; 

//java support
import java.util.*;

//jedit support
import org.gjt.sp.jedit.*;
import org.gjt.sp.util.Log;

//GUI support
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;



/**
This is the main engine behind the PluginHolder, it handles managing plugins, 
instantiating them, and then inserting them within jEdit.

@author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
@version $Id: PluginHolder.java,v 1.11 2000/07/10 03:11:59 akaplan Exp $
*/
public class PluginHolder {
    

    public final static String NAME = "PluginHolder";
    
    private static Vector plugins = new Vector();
    
    

    /**
    Register a plugin with the Plugin Holder.  Only registered plugins are 
    managed.
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolder.java,v 1.11 2000/07/10 03:11:59 akaplan Exp $
    */
    public final static void registerPlugin(PluginHolderEntry plugin) {

        Log.log(Log.DEBUG, PluginHolder.class,
			"registerPlugin -> " + plugin.getName() + " : " + plugin.getClassName() );
        
        //if the developer hasn't explicitly created any actions implicitly create a default one.
        if ( ActionStore.getPluginHolderActions(  plugin.getClassName() ).length == 0 ) {

            if ( plugin.getLabel().indexOf("-menu") >= 0 ) {

                initializeMenu( plugin );

            } else {            

                initializeMenuItem( plugin );

            }
        }

        PluginHolderAction[] actions = ActionStore.getPluginHolderActions(  plugin.getClassName() );
        
        //now add all the necessary plugins
        for (int i = 0; i < actions.length; ++i) {
            jEdit.addAction( actions[i] );
        }
        
    }
    
    /**
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolder.java,v 1.11 2000/07/10 03:11:59 akaplan Exp $
    */
    private static final void initializeMenuItem(PluginHolderEntry entry) {

        PluginHolder.plugins.addElement ( entry );
        
        ActionStore.addPluginHolderAction( new PluginHolderAction( entry ) );

    }
    
    /**
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolder.java,v 1.11 2000/07/10 03:11:59 akaplan Exp $
    */
    private static final void initializeMenu(PluginHolderEntry entry) {
        
        String[] menus = getMenuItems( entry.getLabel() );
        PluginHolder.plugins.addElement ( entry );           

        for (int i = 0; i < menus.length; ++i) {
            
            PluginHolderEntry newEntry = new PluginHolderEntry( entry.getClassName(), menus[i] );
            newEntry.setType(PluginHolderEntry.TYPE_CHILD_MENUITEM);
            newEntry.setParent( entry );
            PluginHolder.plugins.addElement ( newEntry );           

            //FIX ME... should I add the actions here?
            
        }
        
    }
    
    private static final String[] getMenuItems(String menuLabel) {

        String raw = jEdit.getProperty( menuLabel );
        
        StringTokenizer tokens = new StringTokenizer( raw );
        
        Vector v = new Vector();

        while( tokens.hasMoreElements() ) {
            v.addElement( (String)tokens.nextElement() );
        }
        
        String[] labels = new String[ v.size() ];
        v.copyInto(labels);
        
        return labels;
        
    }
    
    /**
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolder.java,v 1.11 2000/07/10 03:11:59 akaplan Exp $
    */
    public final static void registerPlugin( String classname, String label ) {
        registerPlugin( new PluginHolderEntry( classname, label ) );
    }

    

    /**
    Returns all known Holdable Plugins that are currently registered

    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolder.java,v 1.11 2000/07/10 03:11:59 akaplan Exp $
    */
    public final static PluginHolderEntry[] getPlugins() {
        
        PluginHolderEntry[] plugins = new PluginHolderEntry[ PluginHolder.plugins.size() ];
        PluginHolder.plugins.copyInto(plugins);
        return plugins;
        
    }

    
    
    /**
    Given a HoldablePlugin, find its JSplitPane that it is in.
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolder.java,v 1.11 2000/07/10 03:11:59 akaplan Exp $
    */
    private static JSplitPane getSplitPaneForPlugin(HoldablePlugin plugin) {
        if(plugin == null) {
            throw new IllegalArgumentException();
        }

        java.awt.Container c = plugin.getParent();

        while( c != null && !(c instanceof JSplitPane) ) {
            c = c.getParent();
        }

        return (JSplitPane)c;
      }

      
    /**
    Given a view... get it's root component... this can be ither the 
    JEditTextArea or the BufferTabs

    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolder.java,v 1.11 2000/07/10 03:11:59 akaplan Exp $
    */
    public static final JComponent getRootComponent(View view) {


        
        JComponent buffertabs = findComponent( view.getContentPane(),
                                                "org.gjt.sp.jedit.gui.BufferTabs" );
        
        
        
        JComponent textarea = findComponent( view.getContentPane(),
                                             "org.gjt.sp.jedit.textarea.JEditTextArea" );
        

        if (buffertabs != null) {
            return buffertabs;
        } else {
            return textarea;
        }
        
    }



      
    /**
    Add a plugin to a jEdit view.  This method finds the JEditTextArea and then
    places the plugins around it.
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolder.java,v 1.11 2000/07/10 03:11:59 akaplan Exp $
    */
    protected static final void addComponentToView( final View view, 
            final net.sourceforge.jedit.pluginholder.Container plugin ) 
    {

        int location = plugin.getOrientation().getWhere();

		// HACK: until QuickFile has PluginHolder support, we make an exception
		JComponent comp = findComponent(view.getContentPane(),"QuickFilePanel");

		if (comp != null) {
			// We take the JSplitPane above QuickFilePanel
			comp = (JComponent) comp.getParent();
		}
		
		if (comp == null) {
			comp = view.getSplitPane();
		}
		
        if (comp == null) {
            comp = view.getEditPane();
        }

        java.awt.Container parent = comp.getParent();

        JPanel panel = new JPanel(new BorderLayout());
        panel.add(BorderLayout.CENTER, comp);
        comp = panel;
        
        JSplitPane sp = null;
        
        if ( location == Orientation.EAST ) {

            comp.setMinimumSize(new Dimension(0, 0));
            sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, comp, plugin);
            setResizeWeight(sp, 1.0);
            parent.add(BorderLayout.CENTER, sp);


            int div = getDividerLocation(plugin);
            if (div >= 100) {
                sp.setDividerLocation(view.getSize().width - sp.getDividerSize() - div);
            }

        } else if (location == Orientation.NORTH ) {

            comp.setMinimumSize(new Dimension(0, 0));
            sp = new JSplitPane(JSplitPane.VERTICAL_SPLIT, plugin, comp);
            setResizeWeight(sp, 0.0);
            parent.add(BorderLayout.CENTER, sp);


            int div = getDividerLocation(plugin);
            if (div >= 100) {
                sp.setDividerLocation(div);
            }

            
        } else if (location == Orientation.SOUTH ) {

            comp.setMinimumSize(new Dimension(0, 0));
            sp = new JSplitPane(JSplitPane.VERTICAL_SPLIT, comp, plugin );
            setResizeWeight(sp, 1.0);
            parent.add(BorderLayout.CENTER, sp);


            int div = getDividerLocation(plugin);
            if (div >= 100) {
                sp.setDividerLocation(view.getSize().height - sp.getDividerSize() - div);
            }

        
        } else if ( location == Orientation.WEST ) {
            
            comp.setMinimumSize(new Dimension(0, 0));
            sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, plugin, comp);
            setResizeWeight(sp, 0.0);
            parent.add(BorderLayout.CENTER, sp);

            int div = getDividerLocation( plugin );
            if(div >= 100) {
                sp.setDividerLocation(div);
            }
            
        } 


        if (sp != null) {
            //one touch expandable is buggy and also confuses show/hide plugins.
            //there really is no way to figure out when this has been expanded
            sp.setOneTouchExpandable( true );

            //FIX ME:  the border get's too thick when running plugins in all 
            //views
            sp.setBorder( null );
             //sp.setBorder( new EmptyBorder(new Insets( 1,1,1,1 ) ) );
        }

        view.getRootPane().revalidate();
        // view.getContentPane().validate();
    }

      
    /**
    Find an instantiation of the given component within the given container and
    return a reference to it.  If it can't find it then return null.
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolder.java,v 1.11 2000/07/10 03:11:59 akaplan Exp $
    */
    static JComponent findComponent(java.awt.Container container, String classname) {

        Component[] children = container.getComponents();


        for(int i = 0; i < children.length; i++) {

            if( classname.equals( ( children[i].getClass().getName() )) ){

                return (JComponent)children[i];

            } else if ( children[i] instanceof java.awt.Container ) {

                Component ta = findComponent( (java.awt.Container) children[i], classname);

                if(ta != null) { 
                    return (JComponent)ta;
                }

            }

        }

        return null;
    }
      
 
    private static void setResizeWeight(JSplitPane sp, double weight) {
        // This method is actually only available with JDK 1.3 beta or later.
        // Use reflection so this method has no effect on earlier versions.
        final Class[] paramClasses = new Class[] { Double.TYPE };
        try {
            JSplitPane.class.getMethod("setResizeWeight", paramClasses)
                .invoke(sp, new Object[] { new Double(weight) });
        } catch(Exception ex) { }
    }

    /**
    For a given plugin, return where it's divider location should be.
     
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolder.java,v 1.11 2000/07/10 03:11:59 akaplan Exp $
    */
    protected static int getDividerLocation(net.sourceforge.jedit.pluginholder.Container plugin) {
        //FIX ME:  this should be pulled from a resource pool
        return 200;
    }


    /**
    Given some entries, add them to the specified view.
     
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolder.java,v 1.11 2000/07/10 03:11:59 akaplan Exp $
    */
    public static void addPlugins(View view, Profile profile) {

        Orientation north = new Orientation( Orientation.NORTH, Orientation.DEFAULT_DEGREE );
        Orientation east  = new Orientation( Orientation.EAST, Orientation.DEFAULT_DEGREE );
        Orientation west  = new Orientation( Orientation.WEST, Orientation.DEFAULT_DEGREE );
        Orientation south = new Orientation( Orientation.SOUTH, Orientation.DEFAULT_DEGREE );        


        PluginHolder.addPlugins( view, profile.getEntries( north ), north );
        PluginHolder.addPlugins( view, profile.getEntries( west ), west );
        PluginHolder.addPlugins( view, profile.getEntries( east ), east );
        PluginHolder.addPlugins( view, profile.getEntries( south ), south );
        
    }

    /**
    Given some entries, add them to the specified view.
     
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolder.java,v 1.11 2000/07/10 03:11:59 akaplan Exp $
    */
    public static void addPlugins(View view, PluginHolderEntry[] entries, Orientation orientation) {

        for (int i = 0; i < entries.length; ++i) {

            HoldablePlugin plugin;
            

            //create an instance of the plugin...
            try {
                plugin = PluginFactory.getPlugin( entries[i].getClassName(), view, true );
                plugin.getConfig().getOrientation().setWhere( orientation.getWhere() );
                
            } catch (InstantiationException e) {
                e.printStackTrace();
                return;
            }

            
            net.sourceforge.jedit.pluginholder.Container cont = ContainerMonitor.getContainer( view, orientation );
            
            
            if ( cont == null ) {

                cont = new net.sourceforge.jedit.pluginholder.Container( plugin, 
                                              plugin.getConfig().getOrientation(), 
                                              view );

                ContainerMonitor.addContainer( cont );
    
                PluginHolder.addComponentToView( view , cont );
                
            } else {
                cont.addPlugin( plugin );
            }
                                
 
        }
        
    }

    /**
    Remove all plugins within the given view.
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolder.java,v 1.11 2000/07/10 03:11:59 akaplan Exp $
    */
    public static void removePlugins( View view ) {
        
        //first... find the jEdit compoonent above Split
        
        
        //now find a JSplitPane (this should be one of the panels)
        
        
        net.sourceforge.jedit.pluginholder.Container[] containers = ContainerMonitor.getContainers( view );

        //make sure that the given view has some plugins in it
        if ( containers.length == 0 ) {
            Log.log(Log.WARNING, PluginHolder.class, 
				"This view isn't running plugins but you tried to remove some");
            return;
        }
        
        // get the root component
        JComponent rootComp = view.getSplitPane();
        if (rootComp == null) {
            rootComp = view.getEditPane();
        }

		java.awt.Component topComp = rootComp;
		while (topComp != null && topComp.getParent() != view.getContentPane()){
			topComp = topComp.getParent();
		}
		Log.log(Log.DEBUG, PluginHolder.class, "**** [" + topComp + "]");
		
		// HACK: until QuickFile has PluginHolder support, we make an exception
		JComponent quickfileComp = findComponent(view.getContentPane(), "QuickFilePanel");

		JComponent comp = rootComp;
		
		if (quickfileComp != null  && quickfileComp.getParent() instanceof JSplitPane) {
			JSplitPane quickFileSplit = (JSplitPane) quickfileComp.getParent();
			
			if (quickFileSplit.getLeftComponent().getClass().getName().equals("QuickFilePanel")) {
				quickFileSplit.setRightComponent(rootComp);
				// From QuickfilePlugin
				try {
					int div = Integer.parseInt(jEdit.getProperty("quickfile.divider"));
				  	if (div >= 100) {
						quickFileSplit.setDividerLocation(div);
				  	}
				} catch(NumberFormatException nfex) { }
			} else {
				quickFileSplit.setLeftComponent(rootComp);
				// From QuickfilePlugin
				try {
					int div = Integer.parseInt(jEdit.getProperty("quickfile.divider"));
					if (div >= 100) {
						quickFileSplit.setDividerLocation(
							  view.getSize().width
							- quickFileSplit.getDividerSize() - div);
					}
				} catch(NumberFormatException nfex) {}
			}
			comp = quickFileSplit;
		}
		
        java.awt.Container cont = null;
        //remove all the components from jEdit
        for ( int i = 0; i < containers.length; ++i ) {

            cont = containers[i].getParent();
            cont.removeAll();

        }

        //now add the JEditTextArea back into jEdit.
		if (topComp != null) {
			view.getContentPane().remove(topComp);
		}
        view.getContentPane().add( BorderLayout.CENTER, comp );
        view.getRootPane().revalidate();
		
        //remove the containers from the monitor.
        for ( int i = 0; i < containers.length; ++i ) {
            ContainerMonitor.removeContainer( containers[i] );
        }
    }

    public static void dump() {
        /* 
        PluginHolderEntry[] plugins = getPlugins();
    
        for (int i = 0; i < plugins.length; ++i) {
            System.err.println( "Plugin class: " + plugins[i].getClassName() + " -> Label: " + plugins[i].getLabel());
        }
        */
    }
}
