/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package net.sourceforge.jedit.pluginholder;

import org.gjt.sp.jedit.*;
import org.gjt.sp.util.Log;


/**
Defines an entry within plugin holder.  

There are three kinds:

<ul>

<li>
MenuItem: An item off the jEdit plugin pulldown
</li>

<li>
Menu:  An item with children
</li>

<li>
ChildMenuItem:  An Entry off of a menu.  It is important to note that you must
call setParent() on a ChildMenuItem so that PluginHolder knows what the main
application is.  getParent() will return null for all other types of entries.
</li>

</ul>

@author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
@version $Id: PluginHolderEntry.java,v 1.8 2000/07/10 03:11:59 akaplan Exp $
*/
public class PluginHolderEntry {

    public static final int TYPE_MENU           = 1;
    public static final int TYPE_MENUITEM       = 2;
    public static final int TYPE_CHILD_MENUITEM = 3;    

    private String              classname       = null;
    private String              name            = null;
    private String              label           = null;
    private int                 type            = TYPE_MENUITEM;
    private PluginHolderEntry   parent          = null;
    private boolean             standalone      = false;
    
    /**
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolderEntry.java,v 1.8 2000/07/10 03:11:59 akaplan Exp $
    */
    public PluginHolderEntry(String classname, String label) {
        this.classname = classname;
        this.label = label;
    }

    /**
    Return true if this is a menuitem.  A menuitem is a normal jEdit item 
    in a flat listing... 
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolderEntry.java,v 1.8 2000/07/10 03:11:59 akaplan Exp $
    */
    public boolean isMenuItem() {
        return this.type == TYPE_MENUITEM;
    }
   
    /**
    Return true if this is a Menu.  A menu is a jEdit 
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolderEntry.java,v 1.8 2000/07/10 03:11:59 akaplan Exp $
    */
    public boolean isMenu() {
        return this.type == TYPE_MENU;
    }

    /**
    Return true if this is a child menu item.  A child menu item is an item that
    is a child of a menu
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolderEntry.java,v 1.8 2000/07/10 03:11:59 akaplan Exp $
    */
    public boolean isChildMenuItem() {
        return this.type == TYPE_CHILD_MENUITEM;
    }

    /**
    Set the type for this Entry. 
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolderEntry.java,v 1.8 2000/07/10 03:11:59 akaplan Exp $
    */
    public void setType( int type ) {
        this.type = type;
    }
    
    /**
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolderEntry.java,v 1.8 2000/07/10 03:11:59 akaplan Exp $
    */
    public String getName() {
        return jEdit.getProperty( this.getLabel() + ".label");
    }

    /**
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolderEntry.java,v 1.8 2000/07/10 03:11:59 akaplan Exp $
    */
    public String getLabel() {
        return this.label;
    }

    /**
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolderEntry.java,v 1.8 2000/07/10 03:11:59 akaplan Exp $
    */
    public void setLabel(String label) {
        this.label = label;
    }
    
    /**
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolderEntry.java,v 1.8 2000/07/10 03:11:59 akaplan Exp $
    */
    public String getClassName() {
        return this.classname;
    }


    /**
    Compare two PluginHolderEntry(s) for eqaulity.

    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolderEntry.java,v 1.8 2000/07/10 03:11:59 akaplan Exp $
    */
    public boolean equals(Object obj) {

        //if the object isn't an PluginHolderEntry return false
        if ( ! this.getClass().getName().equals( obj.getClass().getName()) ) {
            return false;
        }

        PluginHolderEntry entry = (PluginHolderEntry)obj;
        
        //compare name, classname and label
        
        if (    this.getName() != null &&
                this.getLabel() != null &&
                this.getClassName() != null &&
                this.getName().equals( entry.getName() ) &&
                this.getLabel().equals( entry.getLabel() ) &&
                this.getClassName().equals( entry.getClassName() ) ) {
            
            return true;            
            
        } else {
            return false;
        }
                
        
    }

    /**
    Get the parent for this PluginHolderEntry.  Only valid for TYPE_CHILD_MENUITEM

    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolderEntry.java,v 1.8 2000/07/10 03:11:59 akaplan Exp $
    */
    public PluginHolderEntry getParent() {
        return this.parent;
    }
    
    /**
    Set the parent for this PluginHolderEntry.  Only valid for TYPE_CHILD_MENUITEM

    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolderEntry.java,v 1.8 2000/07/10 03:11:59 akaplan Exp $
    */
    public void setParent(PluginHolderEntry parent) {
        this.parent = parent;
    }

    /**
    <p>
    Requires that only one instance of this HoldablePlugin is allowed to run 
    in each view... IE this plugins "stands alone" in each view.
    </p>
    
    <p>
    Note that this also pays attention to its parent and will return true if 
    this is set to standalone.
    </p>
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolderEntry.java,v 1.8 2000/07/10 03:11:59 akaplan Exp $
    */
    public boolean isRequiredStandalone() {

        if ( this.getParent() != null && 
             this.getParent().isRequiredStandalone() ) {
            return true;
        } else {
            return this.standalone;
        }
    }

    /**
    Set this PluginHolderEntry to required standalone operation.
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolderEntry.java,v 1.8 2000/07/10 03:11:59 akaplan Exp $
    */
    public void setRequiredStandalone( boolean standalone ) {
        this.standalone = standalone;
    }
    
    /**
    Convert the PluginHolderEntry to a string
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolderEntry.java,v 1.8 2000/07/10 03:11:59 akaplan Exp $
    */
    public String toString() {
        return this.getClassName() + " - "  + this.getLabel() + " - " + this.getName();
    }
    
    /**
    Used for debuging.  Take the current plugin and dump it to console.
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: PluginHolderEntry.java,v 1.8 2000/07/10 03:11:59 akaplan Exp $
    */
    public void dump() {
        Log.log(Log.DEBUG, this, "getName:      " + this.getName() );
        Log.log(Log.DEBUG, this, "getLabel:     " + this.getLabel() );
        Log.log(Log.DEBUG, this, "getClassName: " + this.getClassName() );
    }
    
}

