/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package net.sourceforge.jedit.pluginholder;

//jedit support
import org.gjt.sp.jedit.*;
import org.gjt.sp.util.Log;


import java.util.Vector;
import java.util.Hashtable;

/**
Represents a profile for a set of plugins.  There can be multiple profiles 
consisting of multiple profiles.

@author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
@version $Id: Profile.java,v 1.6 2000/07/10 03:11:59 akaplan Exp $
*/
public class Profile {

    public final static String INVALID_ENTRY = "PluginHolder:  WARNING... the PluginHolderEntry given is not registered";
    
    private Hashtable   maps     = new Hashtable();
    private Vector      entries  = new Vector();
    
    private String  name    = null;
    private boolean defaultProfile = false;
    
    
    /**
    Create a reference to a known profile.
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: Profile.java,v 1.6 2000/07/10 03:11:59 akaplan Exp $
    */
    Profile(String name) {
        this.name = name;
    }
    
    
    /**
    Given an entry... and a view... find the plugin for this entry, if it is 
    running.

    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: Profile.java,v 1.6 2000/07/10 03:11:59 akaplan Exp $
    */
    public HoldablePlugin getPlugin( PluginHolderEntry entry, View view ) throws PluginNotFoundException {
        
        //
        if ( ! this.hasEntry( entry ) ) {

            throw new PluginNotFoundException();
        }
       
        //now find the class within the current view.
        HoldablePlugin plugin = (HoldablePlugin)PluginHolder.findComponent( view.getContentPane(), entry.getClassName() );

        if (plugin != null) {
            return plugin;
        }
        
        //default state is that the plugin doesn't exist       
       throw new PluginNotFoundException( "Couldn't find the given plugin" );

    }

    /**
    Determine if this profile has the given entry...
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: Profile.java,v 1.6 2000/07/10 03:11:59 akaplan Exp $
    */
    public boolean hasEntry(PluginHolderEntry entry) {
        
        PluginHolderEntry[] entries = this.getEntries();
        
        for (int i = 0; i < entries.length; ++i) {
            
            if ( entries[i].equals(entry) ) {
                return true;
            }
            
        }
        
        return false;
    }
    
    /**
    Return the name of this profile.
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: Profile.java,v 1.6 2000/07/10 03:11:59 akaplan Exp $
    */
    public String getName() {
        return this.name;
    }

    //FIX ME:  provide a way to get a list of orientations so that you can 
    //go through each one and provide plugins for them
    

    /**
    Given an orientation, return the plugins that are supposed to run there.
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: Profile.java,v 1.6 2000/07/10 03:11:59 akaplan Exp $
    */
    public PluginHolderEntry[] getEntries( Orientation orientation ) {

        Vector v = new Vector();

        PluginHolderEntry[] entries = this.getEntries();

        for ( int i = 0; i < entries.length; ++i ) {
            if ( this.getOrientation( entries[i] ).equals( orientation ) ) {
                v.addElement( entries[i] );
            }
        }

        PluginHolderEntry[] matches = new PluginHolderEntry[v.size()];
        v.copyInto( matches );
        return matches;

    }

    

    /**
    Get all plugin entries for this profile
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: Profile.java,v 1.6 2000/07/10 03:11:59 akaplan Exp $
    */
    public PluginHolderEntry[] getEntries() {
        PluginHolderEntry[] entries = new PluginHolderEntry[this.entries.size()];
        this.entries.copyInto( entries );
        return entries;
    }

    
    /**
    Given an entry (which binary wise might not be identical to the one stored
    locally, the the one stored locally and return.
    

    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: Profile.java,v 1.6 2000/07/10 03:11:59 akaplan Exp $
    */
    public PluginHolderEntry getEntry( PluginHolderEntry entry ) {

        PluginHolderEntry[] entries = this.getEntries();
        
        for (int i = 0; i < entries.length; ++i) {
            if ( entry.getClassName().equals( entries[i].getClassName() ) ) {
                return entries[i];
            }
        }

        return null;
    }


    /**
    Return the number of plugin entries this profile has.
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: Profile.java,v 1.6 2000/07/10 03:11:59 akaplan Exp $
    */
    public int getEntryCount() {
        return this.entries.size();
    }
    
    /**
    Given a PluginHolderEntry determine its orientation under this current 
    profile. or null if the entry does not exist.

    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: Profile.java,v 1.6 2000/07/10 03:11:59 akaplan Exp $
    */
    public Orientation getOrientation( PluginHolderEntry entry ) {


        
        PluginHolderEntry e = getEntry( entry );
        
        if (e == null) {
         
            Log.log(Log.DEBUG, this, INVALID_ENTRY );
            return new Orientation( Orientation.WEST, 0 );

        }
        
        EntryOrientationMap map = (EntryOrientationMap)this.maps.get( e );

        if (map == null) {
            //give back a default orientation
            Log.log(Log.DEBUG, this, INVALID_ENTRY );
            return new Orientation( Orientation.WEST, 0 );
        }

        return map.getOrientation();
        
    }

    /**
    Convert this profile to a String

    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: Profile.java,v 1.6 2000/07/10 03:11:59 akaplan Exp $
    */
    public String toString() {
        return this.getName();
    }
    
    /**
    Remove an entry -> orientation map to the current profile
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: Profile.java,v 1.6 2000/07/10 03:11:59 akaplan Exp $
    */
    public void removeEntryOrientationMap(EntryOrientationMap map) {
        this.maps.remove( map.getEntry() );
        this.entries.removeElement( map.getEntry() );
    }
    
    /**
    Add an entry -> orientation map to the current profile.  You can't add an
    entry directly because it also needs an Orientation to tell it where to run.
    This is why you have to pass it a map.
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: Profile.java,v 1.6 2000/07/10 03:11:59 akaplan Exp $
    */
    public void addEntryOrientationMap(EntryOrientationMap map) {
        this.maps.put( map.getEntry(), map );
        this.entries.addElement( map.getEntry() );
    }

    /**
    Given an entry, remove it, and its orientation from this profile.
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: Profile.java,v 1.6 2000/07/10 03:11:59 akaplan Exp $
    */
    public void removeEntry(PluginHolderEntry entry) {

        PluginHolderEntry real = this.getEntry( entry );
        
        //if real is null then the entry given is bogus
        
        if (real == null) {
            return;
        }

        this.maps.remove( real );
        this.entries.removeElement( real );
        
    }

    /**
    Remove all maps and entries within this profile.

    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: Profile.java,v 1.6 2000/07/10 03:11:59 akaplan Exp $
    */
    public void removeAll() {
        this.maps     = new Hashtable();
        this.entries  = new Vector();
    }
    
}

