/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package net.sourceforge.jedit.pluginholder;

import java.util.*;
import java.io.*;

import org.gjt.sp.jedit.*;
import org.gjt.sp.util.Log;


/**
Contains a set of resources that JProject uses.

@author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
@version $Revision: 1.4 $
*/

public class Resources {

    /**
    This is the resource version... it is used internally by the resources
    to make sure that if I add extra features in the future that break the 
    protocol that I can roll them back.
    */
    public final static int RESOURCE_VERSION        = 0;
    

    /**
    The maximum number of times to ever parse a resource
    */
    public final static int MAX_ITERATIONS          = 1000;

    static private Vector      profiles            = new Vector();    
    

    public static void parse() {

        synchronized (Resources.class) {
    
            //It would be very unusual to have more than 1000 profiles
            
            int i = 0;
            while (i < MAX_ITERATIONS) {
                
                String profile = jEdit.getProperty( "pluginholder.resources.profile." + i );
                
                //if no more profiles were found... continue
                if (profile == null) {
                    break;
                }
                

                Log.log(Log.MESSAGE, Resources.class,
					"Plugin Holder:  Found profile: " + profile );
                Profile prof = new Profile( profile );

                int j = 0;
                while (j < MAX_ITERATIONS) {
                    

                    //now get the entry
                    PluginHolderEntry entry = getEntry( i, j );
                    
                    if (entry == null) {
                        break;
                    }
                    
                    Orientation or = getOrientation( i, j );
                    
                    if ( or == null ) {
                        break;
                    }

                    prof.addEntryOrientationMap( new EntryOrientationMap( entry, or ) );
                    
                    ++j;
                }
    

                //now add the found profile.
                Resources.profiles.addElement( prof );

                ++i;
            }

        }    
    }
    
    /**
    Used internally to get an entry from the resource set

    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: Resources.java,v 1.4 2000/07/10 03:11:59 akaplan Exp $
    */
    private static PluginHolderEntry getEntry(int profile, int entry) {
        
        String classname = jEdit.getProperty( "pluginholder.resources.profile." + profile + ".entry." + entry + ".classname" );

        String label = jEdit.getProperty( "pluginholder.resources.profile." + profile + ".entry." + entry + ".label" ); 


        //make sure this was a valid entry
        if (classname == null || label == null ) {
            return null;
        }

        Log.log(Log.MESSAGE, Resources.class,
			"Plugin Holder:  Found profile entry: " + classname );

        
        return new PluginHolderEntry( classname, label );
        
    }



    /**
    Used internally to get an Orientation from the resource set

    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: Resources.java,v 1.4 2000/07/10 03:11:59 akaplan Exp $
    */
    private static Orientation getOrientation(int profile, int entry) {
        
        String where = jEdit.getProperty( "pluginholder.resources.profile." + profile + ".entry." + entry + ".where" );

        String degree = jEdit.getProperty( "pluginholder.resources.profile." + profile + ".entry." + entry + ".degree" );

        //make sure this was a valid entry
        if (where == null || degree == null ) {
            return null;
        }

        Log.log(Log.MESSAGE, Resources.class, "Plugin Holder:  Found profile orientation: "
			+ where + "," + degree );

        
        int w = Orientation.WEST;
        int d = 0;

        try {
            w = Integer.parseInt( where );
            d = Integer.parseInt( degree );

        } catch (NumberFormatException e) {
            e.printStackTrace();
        }
    
        return new Orientation( w, d );
        
    }


    
    /**
    Take thes Project and save it to disk....

    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: Resources.java,v 1.4 2000/07/10 03:11:59 akaplan Exp $
    */
    static synchronized public void save() {
           
        /*
        FORMAT IS THE FOLLOWING

        profile.<PROFILE_KEY>=Test
        profile.<PROFILE_KEY>.entry.<ENTRY_KEY>=projectviewer.ProjectViewer
        profile.<PROFILE_KEY>.entry.<ENTRY_KEY>.where=0
        profile.<PROFILE_KEY>.entry.<ENTRY_KEY>.degree=0
        */

        Resources.clearAllResources();

        //set the resource version
        
        
        jEdit.setProperty( "pluginholder.resources.version", new Integer( RESOURCE_VERSION ).toString() );
        
        Profile[] profiles = ProfileFactory.getProfiles();
        
        for ( int i = 0; i < profiles.length; ++i ) {
            
            Log.log(Log.MESSAGE, Resources.class, "PluginHolder: saving profile: "
				+ profiles[i].getName() );

            jEdit.setProperty( "pluginholder.resources.profile." + i,  profiles[i].getName() );
            
            PluginHolderEntry[] entries = profiles[i].getEntries();
            
            for (int j = 0; j < entries.length; ++j) {
                
                if ( profiles[i].hasEntry( entries[j] ) ) {

                    //Only save if the profile has support for the given entry

                    Log.log(Log.MESSAGE, Resources.class,
						"PluginHolder: saving entry: " + entries[j].getClassName() );
                    
                    jEdit.setProperty( "pluginholder.resources.profile." + i + ".entry." + j + ".classname", 
                                       entries[j].getClassName() );

                    jEdit.setProperty( "pluginholder.resources.profile." + i + ".entry." + j + ".label", 
                                       entries[j].getLabel() );
                
                    Orientation or = profiles[i].getOrientation( entries[j] );
                
                    jEdit.setProperty( "pluginholder.resources.profile." + i + ".entry." + j + ".where",  
                                        new Integer( or.getWhere() ).toString() );

                    jEdit.setProperty( "pluginholder.resources.profile." + i + ".entry." + j + ".degree", 
                                        new Integer( or.getDegree() ).toString() );

                } 

            }
            
        }

    }

    /**
    Get all profiles stored within this resource set..
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: Resources.java,v 1.4 2000/07/10 03:11:59 akaplan Exp $
    */
    public static Profile[] getProfiles() {

        Profile[] profiles = new Profile[ Resources.profiles.size() ];
        Resources.profiles.copyInto( profiles );
        return profiles;
    }


    /**
    If a property exists within jEdit... clear it
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: Resources.java,v 1.4 2000/07/10 03:11:59 akaplan Exp $
    */
    private static void clearProperty(String property) throws NoResourceException {

        if (jEdit.getProperty( property ) != null) {
            jEdit.unsetProperty( property );
        } else {
            throw new NoResourceException();
        }

    }

    /**
    If any resources are in disk... unset them.
    
    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: Resources.java,v 1.4 2000/07/10 03:11:59 akaplan Exp $
    */
    private static void clearAllResources() {
 

        synchronized (Resources.class) {

            try { 
            
                //It would be very unusual to have more than 1000 profiles
    
                
                int i = 0;
                while (i < MAX_ITERATIONS) {
                    
                    clearProperty( "pluginholder.resources.profile." + i );
    
                    int j = 0;
                    while (j < MAX_ITERATIONS) {
                        
    
                        clearProperty( "pluginholder.resources.profile." + i + ".entry." + j + ".classname" );

                        clearProperty( "pluginholder.resources.profile." + i + ".entry." + j + ".label" ); 

                        clearProperty( "pluginholder.resources.profile." + i + ".entry." + j + ".where" );

                        clearProperty( "pluginholder.resources.profile." + i + ".entry." + j + ".degree" );

                        
                        ++j;
                    }
    
                    ++i;
                }
            } catch (NoResourceException e) {
                
                //Don't handle anything here... this is a normal situation
                //when clearProperty doesn't find it's target
                
            }
    
        }    

       
        
    }
    
    /**
    Get the resources in a raw format... this should never be used outside of
    the PluginHolder packages

    @author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
    @version $Id: Resources.java,v 1.4 2000/07/10 03:11:59 akaplan Exp $
    */
    static Vector getResources() {
        
        synchronized( Resources.profiles ) {
            return Resources.profiles;
        }
        
    }
    
    
    //bootstrap for testing...
    public static void main(String args[]) {
        parse();
    }

    

    
    
}


/**
Thrown if the resource doesn't exist

@author <A HREF="mailto:burton@relativity.yi.org">Kevin A. Burton</A>
@version $Id: Resources.java,v 1.4 2000/07/10 03:11:59 akaplan Exp $
*/
class NoResourceException extends Exception {
    
    public NoResourceException() {
        super();
    }
    
}
